# -*- coding: utf-8 -*-

"""A collection of string format functions derived form the class “Functions()”
"""

from tmep.functions import Functions

f = Functions()

alpha = f.tmpl_alphanum
alphanum = f.tmpl_alphanum
asciify = f.tmpl_asciify
delchars = f.tmpl_delchars
deldupchars = f.tmpl_deldupchars
first = f.tmpl_first
initial = f.tmpl_initial
left = f.tmpl_left
lower = f.tmpl_lower
num = f.tmpl_num
replchars = f.tmpl_replchars
right = f.tmpl_right
sanitize = f.tmpl_sanitize
shorten = f.tmpl_shorten
time = f.tmpl_time
title = f.tmpl_title
upper = f.tmpl_upper
